import requests

class RdapClient:
    
    def __init__(self) -> None:
        self._client = requests.Session()
        self._headers = {
            "Accept" : "application/json",
            "content-type" : "application/json",
        }
    
    def get(self, link:str) -> dict:
        response = self._client.get(
            url=link,
            headers=self._headers
        )
        
        # TODO I have to implement a retry policy and strategy.
        if response.status_code == 404:
            context = {
                "status" : True,
                "content" : {},
            }
        else:
            context = {
                "status" : False,
                "content" : response.json(),
            }

        return context



import validators
from .exceptions import (
    DomainWithSubdomain,
    DomainValidationError,
    DomainWithHttp,
)
from rdap.common.utils import (
    get_suffix,
    get_domain,
    get_subdomain,
)


def _has_subdomain(domain:str) -> None:
    """In charge of validate if the domain contains a subdomain

    Args:
        domain (str): [Some domain name. I.e example.com]

    Raises:
        DomainWithSubdomain: [
            Raised when the domain validation regex has found a match
            with a subdomain passed by input with the domain.
        ]
    """
    subdomain = get_subdomain(domain)
    if subdomain:
        raise DomainWithSubdomain(
            (
                "Only admits valid domain names, subdomain are not included. "
                "Try deleting '{0}.' and try again please.".format(
                    domain.split(".")[0]
                )
            )
        )

def _has_http(domain:str) -> None:
    """In charge of validate if the domain contains http or https strings

    Args:
        domain (str): [Some domain name. I.e example.com]

    Raises:
        DomainWithHttp: [
            Raised when the domain validation found http or https
            in the domain input.
        ]
    """

    if "http" in domain:
        raise DomainWithHttp(
            (
                "Only admits valid domain names. "
                "Try deleting '{0}://' and try again please.".format(
                    domain.split("://", 1)[0]
                )
            )
        )

def _regex_domain(domain:str) -> None:
    """In charge of validate if the domain matches with the standart
    domain regex (use of external library - 'Validators')

    Args:
        domain (str): [Some domain name. I.e example.com]

    Raises:
        DomainValidationError: [
            Raised when the domain validation regex does not match
            any coincidence with the domain.
        ]
    """

    if not validators.domain(domain):
        raise DomainValidationError(
            (
                "Only admits valid domain names. "
                "Please try only typing {0}.{1} and try again.".format(
                    get_domain(domain), get_suffix(domain)
                )
            )
        )

def domain_validator(domain:str) -> str:
    """Validator to check if the input domain is valid or not.

    Args:
        domain (str): [Some domain name. I.e example.com]

    Returns:
        str: [return back the domain if it is all ok]
    """
    _has_http(domain)
    _has_subdomain(domain)
    _regex_domain(domain)
    
    return domain



import click
from rdap.commands.gather import gather
from rdap.commands.history import history
from rdap.commands.check import check
from rdap.commands.version import version



@click.group()
@click.pass_context
def cli(ctx) -> None:
    pass

cli.add_command(gather)
cli.add_command(check)
cli.add_command(history)
cli.add_command(version)

if __name__ == "__main__":
    cli()



import os
import click
from rdap.settings import CACHE_FILE_PATH
from rdap.commands.history.utils import (
    get_record,
    generate_table,
)
from rdap.common.utils import (
    format_domain_output,
    load_file_data,
)
from rdap.common.constants import (
    MessageColors,
)
from rdap.common.save import Save


@click.group(name="history", invoke_without_command=True)
@click.pass_context
def history(ctx):
    """Returns a table with the history of the last searches."""

    if ctx.invoked_subcommand is None:
        generate_table()


@history.command(name="detail")
@click.argument("id")
def detail(id):
    """Returns the detail of the specific ID."""

    output = get_record(id)
    if output:
        message = format_domain_output(output)
        click.echo(message)


@history.command(name="clear")
def clear():
    """Clean history."""

    try:
        os.remove(CACHE_FILE_PATH)
    except FileNotFoundError:
        return click.echo(
            click.style(
                "[INFO] - You have no history to delete.",
                fg=MessageColors.YELLOW,
                bold=True,
            )
        )

    click.echo(
        click.style(
            "[DONE] - History was cleaned.",
            fg=MessageColors.GREEN,
            bold=True,
        )
    )


@history.command(name="download")
@click.argument("filename")
def download(filename):
    """Download the history into a file."""

    try:
        _history = load_file_data(CACHE_FILE_PATH)
    except FileNotFoundError:
        return click.echo(
            click.style(
                "[INFO] - Nothing to download.",
                fg=MessageColors.YELLOW,
                bold=True,
            )
        )

    Save().save_harvest(filename, _history)



import os

# General Settings
DNS_FILE_CHECK_INTERVAL = 7  # Number of days to evaluate until check again the dns file
CACHE_LIMIT_RECORDS = (
    25  # Define the max number of records that will be saved in the cache.json file
)
RDAP_CACHE_FILENAME = (
    "cache.json"  # Name of the file where the history is going to be saved
)
RDAP_DNS_FILENAME = (
    "dns.json"  # Name of the file where the hosts and tlds are going to be saved.
)
UNDEFINED_DATA = (
    "Undefined"  # Name of the variable to show if some data is not present.
)

# Files
BASE_DIR = os.path.dirname(
    os.path.dirname(os.path.abspath(__file__))
)  # Base dir of the project
CACHE_FILE_PATH = os.path.join(BASE_DIR, RDAP_CACHE_FILENAME)



class MessageColors:
    BLACK = "black"
    GREEN = "green"
    RED = "red"
    BLUE = "blue"
    YELLOW = "yellow"
    CYAN = "cyan"
    WHITE = "white"


class RdapDomainEvents:
    REGISTRATION = "registration"
    EXPIRATION = "expiration"
    LAST_CHANGED = "last changed"
    LAST_CHANGED_RDAP = "last update of RDAP database"


class TextFormatConstants:
    TEXT = "txt"
    JSON = "json"


class DomainAvailability:
    AVAILABLE = "AVAILABLE"
    UNAVAILABLE = "UNAVAILABLE"

    availability_color_map = {
        AVAILABLE : MessageColors.GREEN,
        UNAVAILABLE : MessageColors.RED
    }


from rdap.common.constants import MessageColors
import click
from rdap.services.rdap import RdapApi
from rdap.commands.gather.utils import domain_validator
from rdap.common.utils import format_domain_output
from rdap.commands.gather.exceptions import (
    DomainWithSubdomain,
    DomainWithHttp,
    DomainValidationError,
)
from rdap.common.save import Save


@click.command()
@click.argument("domain", nargs=1)
@click.option(
    "-s",
    "--file",
    "filename",
    help=(
        "Give a file format and the result is going to be saved there. "
        "I.e 'my_file.json' or 'my_file.txt'"
    ),
)
def gather(domain: str, filename: str) -> None:
    """Gather the domain information and prints in the shell.
    Give a valid domain name. In example: 'google.com', 'mydomain.net', etc.
    """

    try:
        domain_validator(domain)
    except (DomainWithSubdomain, DomainWithHttp, DomainValidationError) as ex:
        return click.echo(
            click.style(
                f"[ERROR] {ex}",
                fg=MessageColors.RED,
                bold=True,
            )
        )

    schema = RdapApi(domain).query()

    if filename:
        return Save().save_harvest(filename, schema)

    message = format_domain_output(schema)

    click.echo(message)



import os
import click
import json
import dateutil.parser
import tldextract
from datetime import datetime
from rdap.settings import UNDEFINED_DATA
from rdap.common.exceptions import (
    NotSupportedFileFormat,
)
from rdap.common.constants import (
    TextFormatConstants,
    MessageColors,
    DomainAvailability,
)

AVAILABLE_EXTENCION = (
    TextFormatConstants.JSON,
    TextFormatConstants.TEXT,
)


def load_file_data(filename: str) -> dict or str:
    """Receive a filename, try to check if the format is valid and
    return the respective data according to its format.

    Args:
        filename (str): [Filename. I.e: my_file.json]
    Raises:
        FileNotFoundError: [raise if the file is not found]
    Returns:
        dict or str: [
            deppending on the file format return
            a dict object or a str
        ]
    """

    if not os.path.isfile(filename):
        raise FileNotFoundError

    with open(filename, "r") as output:
        if filename.endswith(TextFormatConstants.JSON):
            data = json.load(output)
        elif filename.endswith(TextFormatConstants.TEXT):
            data = output.read()

    return data


def save_file_data(data: dict, filename: str, _type: str) -> None:
    """Save the data in the corresponding file.

    Args:
        data (dict): [Data to be saved]
        filename (str): [Filename. I.e: my_file.json]
        _type (str): [Define the file type]

    Raises:
        NotSupportedFileFormat: [
            This exception occurs when the user entersa text
            format that is not valid or is not supported by the cli.
        ]
    """

    if _type not in AVAILABLE_EXTENCION:
        raise NotSupportedFileFormat(f"The extension '.{_type}' is not supported yet")

    with open(filename, "w") as input:

        if _type == TextFormatConstants.JSON:
            json.dump(data, input)

        elif _type == TextFormatConstants.TEXT:
            data = convert_dict_into_txt(data)
            input.write(data)


def string_to_datetime(date: str) -> datetime:
    """Convert a datetime stringified into a datetime object

    Args:
        date (str): [Datetime as string]

    Returns:
        datetime: [Datetime as datetime.datetime]
    """

    if date:
        return dateutil.parser.parse(date).replace(tzinfo=None)
    return date


def datetime_to_string(date: datetime) -> str:
    """Convert a datetime object into a datetime stringified.

    Args:
        date (datetime): [Datetime object]

    Returns:
        str: [Datetime as String]
    """

    if date:
        return date.strftime("%Y-%m-%d %H:%M")
    return date


def get_subdomain(domain: str) -> str:
    """Receive a domain and parse it returning the subdomain
    based on a regex pattern. (external library)

    Args:
        domain (str): [Receive a domain. I.e example.com]

    Returns:
        str: [
            Return only the subdomain of the domain.
            I.e if blog.example.com -> blog
        ]
    """
    try:
        return tldextract.extract(domain).subdomain
    except TypeError:
        return ""


def get_domain(domain: str) -> str:
    """Receive a domain and parse it returning the domain
    based on a regex pattern. (external library)

    Args:
        domain (str): [Receive a domain. I.e example.com]

    Returns:
        str: [
            Return only the subdomain of the domain.
            I.e if blog.example.com -> example
        ]
    """
    try:
        return tldextract.extract(domain).domain
    except TypeError:
        return ""


def get_suffix(domain: str) -> str:
    """Receive a domain and parse it returning the suffix
    based on a regex pattern. (external library)

    Args:
        domain (str): [Receive a domain. I.e example.com]

    Returns:
        str: [
            Return only the suffix of the domain.
            I.e if blog.example.com -> com
        ]
    """
    try:
        return tldextract.extract(domain).suffix
    except TypeError:
        return ""


def form_hostname(data: str) -> str:
    """In charge of forming a hostname based on the data
    received. If data is true return a descent hostname.
    I.e https://example.com

    Args:
        data (str): [receive a domain name or host name]

    Returns:
        str: [a more beautiful hostname]
    """
    domain = get_domain(data)
    suffix = get_suffix(data)

    if domain != "" and suffix != "":
        return "https://{0}.{1}/".format(domain, suffix)
    return UNDEFINED_DATA


def get_availability(data: dict) -> str:
    """This is to avoid repeating the code so many times.
    It is mainly used to define the domain availability status and color it.

    Args:
        data (dict): [dict with some data from the context]

    Returns:
        str: [return a stringified version of the domain's availability]
    """

    if data.get("status"):
        availability = click.style(
            DomainAvailability.AVAILABLE,
            fg=DomainAvailability.availability_color_map.get(
                DomainAvailability.AVAILABLE
            ),
            bold=True,
        )
    else:
        availability = click.style(
            DomainAvailability.UNAVAILABLE,
            fg=DomainAvailability.availability_color_map.get(
                DomainAvailability.UNAVAILABLE
            ),
            bold=True,
        )

    return availability


def format_domain_output(data: dict) -> str:
    """This is to avoid repeating the code so many times.
    It is mainly used to format a user-friendly view when
    it is called from the gather or the check commands.

    Args:
        data (dict): [dict with data from the context]

    Returns:
        str: [return a stringified version of the domain's data]
    """

    is_available = get_availability(data)
    data = data["content"]
    dns = " \n\t\t    ".join(data.get("dns").split(","))
    message = f"""
        Domain: {
            click.style(data.get("domain").upper(), fg=MessageColors.WHITE, bold=True)
        }
        Status: {is_available}

        Nameservers: {
            click.style(dns, fg=MessageColors.WHITE, bold=True)
        }

        Create date: {
            click.style(
                data.get(
                    "create_at", UNDEFINED_DATA
                ), fg=MessageColors.WHITE, bold=True
            )
        }
        Expire date: {
            click.style(
                data.get(
                    "expire_at", UNDEFINED_DATA
                ), fg=MessageColors.WHITE, bold=True
            )
        }
        Update date: {
            click.style(
                data.get(
                    "update_at", UNDEFINED_DATA
                ), fg=MessageColors.WHITE, bold=True
            )
        }
        Update date (RDAP): {
            click.style(
                data.get(
                    "updata_at_rdap", UNDEFINED_DATA
                ), fg=MessageColors.WHITE, bold=True
            )
        }

        Entity: {
            click.style(
                data.get(
                    "entity", UNDEFINED_DATA
                ), fg=MessageColors.WHITE, bold=True
            )
        }
        Name: {
            click.style(
                data.get(
                    "name", UNDEFINED_DATA
                ), fg=MessageColors.WHITE, bold=True
            )
        }
        Registrant ID: {
            click.style(
                data.get(
                    "registrant_id", UNDEFINED_DATA
                ), fg=MessageColors.WHITE, bold=True
            )
        }
    """
    return message


def convert_dict_into_txt(data: dict) -> str:
    """This function recursively converts the contents of a dictionary
    to plain text and then be delivered in a text file.

    Args:
        data (dict): [The data to be converted into txt]

    Returns:
        str: [The converted dictionary into text]
    """

    line = ""

    if isinstance(data, list):
        for elements in data:
            for key, value in elements.items():

                if isinstance(value, dict):
                    line += convert_dict_into_txt(value)
                else:
                    line += f"{key}: {value}\n"
    else:
        for key, value in data.items():

            if isinstance(value, dict):
                line += convert_dict_into_txt(value)
            else:
                line += f"{key}: {value}\n"

    return line



from rdap.common.constants import MessageColors
import click

@click.group(name="settings", invoke_without_command=True)
@click.pass_context
def settings(ctx):
    """ Print the CLI settings. """

    if ctx.invoked_subcommand is None:
        print("SETTINGS WIP")


@settings.command()
def set():
    """
    Modify some params of the CLI
    """
    print("SET WIP")

@settings.command()
def show():
    """
    Show the available params of the CLI.
    Their status and values.
    """    
    print("SHOW WIP")


class DomainValidationError(Exception):
    """
    Raised when the domain validation regex does not match
    any coincidence with the domain.
    """

    def __init__(self, message):
        super().__init__(message)


class DomainWithSubdomain(DomainValidationError):
    """
    Raised when the domain validation regex has found a match
    with a subdomain passed by input with the domain.
    """


class DomainWithHttp(DomainValidationError):
    """
    Raised when the domain validation found http or https
    in the domain input.
    """



import os
import click
from typing import Any
from rdap.common.utils import save_file_data
from rdap.common.constants import (
    MessageColors,
    TextFormatConstants,
)
from rdap.common.exceptions import NotSupportedFileFormat


class Save:
    FILE_TYPE = None
    AVAILABLE_EXTENCION = (
        TextFormatConstants.JSON,
        TextFormatConstants.TEXT,
    )

    @classmethod
    def _validate_format(cls, filename: str) -> type:
        """Validate if the fileformat is supported by the CLI.

        Args:
            filename (str): [Filename. I.e myfile.json]

        Raises:
            NotSupportedFileFormat: [
                Raised when the CLI found a non supported file format.
            ]
        """
        extention = filename.split(".", 1)[1]
        if not filename.endswith(cls.AVAILABLE_EXTENCION):
            raise NotSupportedFileFormat(
                f"The fileformat [{extention}] is not supported yet."
            )
        cls.FILE_TYPE = extention

    @classmethod
    def save_harvest(cls, filename: str, content: Any):
        """Save the content into the filename if the file format
        is supported.

        Args:
            filename (str): [Filename. I.e myfile.json]
            content (Any): [The content to be saved.]
        """

        try:
            cls._validate_format(filename)
        except NotSupportedFileFormat as ex:
            return click.echo(
                click.style(f"[ERROR] - {ex}", fg=MessageColors.RED, bold=True)
            )

        save_file_data(
            content,
            filename,
            cls.FILE_TYPE,
        )

        return click.echo(
            click.style(
                f"[DONE] - File saved successfully in {os.getcwd()}/{filename}",
                fg=MessageColors.GREEN,
                bold=True,
            )
        )



import os
import uuid
from datetime import datetime
from rdap.common.endpoints import RDAP_DNS
from rdap.services.rdap_client import RdapClient
from rdap.common.utils import (
    load_file_data,
    save_file_data,
    string_to_datetime,
    datetime_to_string,
    form_hostname,
)
from rdap.common.constants import (
    RdapDomainEvents,
    TextFormatConstants,
)
from rdap.settings import (
    BASE_DIR,
    RDAP_DNS_FILENAME,
    UNDEFINED_DATA,
    CACHE_FILE_PATH,
    CACHE_LIMIT_RECORDS,
)


def save_history(method):
    """
    Decorator designed to save the query into a json-file when 'gather' command
    or 'check' were called.

    Args:
        method ([type]): Receive classmethod.
    """

    def save(cls):
        data = method(cls)

        if not os.path.isfile(CACHE_FILE_PATH):
            save_file_data([], CACHE_FILE_PATH, TextFormatConstants.JSON)

        data["id"] = str(uuid.uuid4())[:8]
        data["timestamp"] = datetime_to_string(datetime.now())
        output = load_file_data(CACHE_FILE_PATH)

        if len(output) >= CACHE_LIMIT_RECORDS:
            output.pop(0)

        output.append(data)

        save_file_data(output, CACHE_FILE_PATH, TextFormatConstants.JSON)
        return method(cls)

    return save


class RdapApi:
    CLIENT = RdapClient()
    DNS_FILE_DIR = os.path.join(BASE_DIR)
    DNS_FILE_PATH = os.path.join(BASE_DIR, RDAP_DNS_FILENAME)
    IS_PART_OF_RDAP_PROTOCOL = False
    AVAILABLE_TO_REGISTER = False
    DOMAIN_HOST = None

    def __init__(self, domain: str) -> None:
        self.domain = domain

        if not os.path.isfile(self.DNS_FILE_PATH):
            response = self.CLIENT.get(RDAP_DNS).get("content")
            save_file_data(response, self.DNS_FILE_PATH, TextFormatConstants.JSON)

        else:
            file_date = os.path.getmtime(self.DNS_FILE_PATH)
            file_date = datetime.fromtimestamp(file_date)

            if (datetime.now() - file_date).days > 7:
                response = self.CLIENT.get(RDAP_DNS).get("content")
                save_file_data(response, RDAP_DNS_FILENAME, TextFormatConstants.JSON)

        self.DOMAIN_HOST = self._get_host(self.domain)

    @classmethod
    def _get_host(cls, domain: str) -> None:
        """
        In charge of getting the domain host and determine
        if the domain is part of the RDAP protocol or not.

        Args:
            domain (str): [Domain name. I.e: 'example.com']

        """

        context = load_file_data(cls.DNS_FILE_PATH)

        for tld, service in context.get("services"):
            if domain.endswith(tuple(tld)):
                cls.IS_PART_OF_RDAP_PROTOCOL = True
                cls.DOMAIN_HOST = "{0}domain/{1}".format(service[0], domain)

    @classmethod
    def _get_scheme(cls, domain: str) -> dict:
        """
        Returns the scheme that will later be rendered in the commands.

        Args:
            domain (str): [Domain name. I.e: 'example.com']
        Returns:
            dict: [A full scheme of the info to be showed on screen later]
        """

        schema = {
            "is_rdap": cls.IS_PART_OF_RDAP_PROTOCOL,
            "status": False,
            "domain": domain,
            "host": form_hostname(cls.DOMAIN_HOST),
            "query_host": cls.DOMAIN_HOST or UNDEFINED_DATA,
            "content": {
                "domain": domain,
                "dns": UNDEFINED_DATA,
                "create_at": UNDEFINED_DATA,
                "expire_at": UNDEFINED_DATA,
                "update_at": UNDEFINED_DATA,
                "update_at_rdap": UNDEFINED_DATA,
                "entity": UNDEFINED_DATA,
                "registrant_id": UNDEFINED_DATA,
                "name": UNDEFINED_DATA,
            },
        }

        return schema

    @classmethod
    def _get_nameservers(cls, context_data: dict) -> list:
        """
        Return a list of nameservers related to the domain.

        Args:
            context_data (dict): [
                The complete payload comming from the host
            ]
        Returns:
            list: [a list of nameservers]
        """

        dns_list = []
        if "nameservers" in context_data:
            while len(dns_list) < (len(context_data["nameservers"])):
                dns_list.append(
                    context_data["nameservers"][len(dns_list)]["ldhName"].lower()
                )

        return dns_list

    @classmethod
    def _get_events(cls, context_data: dict) -> dict:
        """
        In charge of parsing the context data and look up for the
        domain events (expiration date, creation date, etc.)

        Args:
            context_data (dict): [
                The complete payload comming from the host
            ]
        Returns:
            dict: [A dictionary with the timestamps of the domain]
        """

        events = {}
        if context_data:
            for event in context_data["events"]:

                if event["eventAction"] == RdapDomainEvents.REGISTRATION:
                    events["create_at"] = datetime_to_string(
                        string_to_datetime(event["eventDate"])
                    )

                elif event["eventAction"] == RdapDomainEvents.EXPIRATION:
                    events["expire_at"] = datetime_to_string(
                        string_to_datetime(event["eventDate"])
                    )

                elif event["eventAction"] == RdapDomainEvents.LAST_CHANGED:
                    events["update_at"] = datetime_to_string(
                        string_to_datetime(event["eventDate"])
                    )

                elif event["eventAction"] == RdapDomainEvents.LAST_CHANGED_RDAP:
                    events["update_at_rdap"] = datetime_to_string(
                        string_to_datetime(event["eventDate"])
                    )

        return events

    @classmethod
    def _get_owner_data(cls, contex_data: dict, domain: str) -> dict:
        """
        In charge of parsing the context data and look up for the
        domain specific data, making focus on the owners data.

        Args:
            context_data (dict): [
                The complete payload comming from the host
            ]
            domain (str): [Domain name. I.e: 'example.com']

        Returns:
            dict: [Returns a dictionary with the domain owner data]
        """

        data = {}
        if contex_data:
            if domain.endswith(".ar"):
                data["entity"] = "Nic Argentina"
                data["id"] = contex_data["entities"][0]["handle"]

                url = contex_data["entities"][0]["links"][0]["href"]
                name = cls.CLIENT.get(url).get("content")
                data["name"] = name["vcardArray"][1][1][-1]

            else:
                data["entity"] = contex_data["entities"][0]["vcardArray"][1][1][-1]

        return data

    @classmethod
    def _get_context_data(cls) -> dict:
        """
        Make a request into the domain host and retrive all the
        context data required to be parsed latter.

        Returns:
            dict: [Return the full context data from the host]
        """

        return cls.CLIENT.get(cls.DOMAIN_HOST)

    @save_history
    def query(self) -> dict:
        """
        In charge of gathering all the information about the domain
        and putting together a context to be delivered later.

        Returns:
            dict: [Return the formated sheme with the domain information]
        """
        schema = self._get_scheme(self.domain)

        if not self.IS_PART_OF_RDAP_PROTOCOL:
            return schema

        context = self._get_context_data()
        if context.get("status"):
            schema["status"] = True
            return schema

        dates = self._get_events(context.get("content"))
        owner_data = self._get_owner_data(context.get("content"), self.domain)

        schema["content"]["dns"] = ", ".join(
            self._get_nameservers(context.get("content"))
        )
        schema["content"]["create_at"] = dates.get("create_at" or UNDEFINED_DATA)
        schema["content"]["expire_at"] = dates.get("expire_at" or UNDEFINED_DATA)
        schema["content"]["update_at"] = dates.get("update_at" or UNDEFINED_DATA)
        schema["content"]["update_at_rdap"] = dates.get(
            "update_at_rdap" or UNDEFINED_DATA
        )
        schema["content"]["entity"] = owner_data.get("entity" or UNDEFINED_DATA)
        schema["content"]["registrant_id"] = owner_data.get("id" or UNDEFINED_DATA)
        schema["content"]["name"] = owner_data.get("name" or UNDEFINED_DATA)

        return schema



class FileError(Exception):
    def __init__(self, message):
        super().__init__(message)


class FileDoesNotExist(FileError):
    """
    This exception is raised when the file is not found.
    """


class NotSupportedFileFormat(FileError):
    """
    Raised when the CLI found a non supported file format.
    """



import click
from rdap.services.rdap import RdapApi
from rdap.common.constants import MessageColors, DomainAvailability
from rdap.commands.gather.utils import domain_validator
from rdap.common.utils import (
    form_hostname,
    get_availability,
) 
from rdap.settings import UNDEFINED_DATA
from rdap.commands.gather.exceptions import (
    DomainWithSubdomain,
    DomainWithHttp,
    DomainValidationError,    
)

@click.command()
@click.argument("domain", nargs=1)
def check(domain) -> None:
    """ Check if the domain is available or not. """

    try:
        domain_validator(domain)
    except (
        DomainWithSubdomain,
        DomainWithHttp,
        DomainValidationError,
    ) as ex:
        return click.echo(
            click.style(
                f"[ERROR] {ex}",
                fg=MessageColors.RED,
                bold=True,
            )
    )

    schema = RdapApi(domain).query()
    is_available = get_availability(schema)
    message = f"""
    Domain: {click.style(domain.upper(), MessageColors.WHITE, bold=True)}
    Status: {is_available}
    Rdap Host: {form_hostname(schema.get("query_host", UNDEFINED_DATA))}
    Query host: {schema.get("query_host", UNDEFINED_DATA)}
    """
    click.echo(message)



from os import path
from setuptools import setup, find_packages

_version = "1.1.1"
_cwd = path.abspath(path.dirname(__file__))


def get_packages() -> list:
    """simple function designed to return a list
    of packages involved in our CLI

    Returns:
        list: [package list]
    """

    root = ["rdap"]
    package_list = ["rdap.{0}".format(package) for package in find_packages(root[0])]
    return root + package_list


with open(path.join(_cwd, "README.md"), encoding="utf-8") as f:
    _long_description = f.read()

setup(
    name="rdap-cli",
    version=_version,
    author="Alexei Popov",
    author_email="alexei.popov.cli@gmail.com",
    long_description=_long_description,
    long_description_content_type="text/markdown",
    entry_points={
        "console_scripts": [
            "rdap=rdap.rdap:cli",
        ]
    },
    description=(
        "A simple CLI designed to offer a quick way of gathering domain"
        "data information."
    ),
    url="https://github.com/alexeipopov95/rdap-cli",
    download_url=(
        f"https://github.com/alexeipopov95/rdap-cli/archive/refs/tags/{_version}.tar.gz"
    ),
    keywords=["rdap", "cli", "whois", "registration-date", "data", "domain"],
    python_requires=">=3.6",
    install_requires=[
        "black==21.6b0",
        "click==8.0.1",
        "flake8==3.9.2",
        "pre-commit==2.13.0",
        "python-dateutil==2.8.1",
        "PyYAML==5.4.1",
        "requests==2.25.1",
        "tabulate==0.8.9",
        "tldextract==3.1.0",
        "validators==0.18.2",
    ],
    packages=get_packages(),
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Software Development",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



import click
import pkg_resources
from rdap.common.constants import MessageColors


@click.command(name="version")
def version():
    """
    Show the project version.
    """

    version = pkg_resources.require("rdap-cli")[0].version
    click.echo(click.style(str(version), fg=MessageColors.YELLOW, bold=True))



import click
from tabulate import tabulate
from typing import Union
from rdap.common.utils import (
    load_file_data
)
from rdap.common.constants import (
    MessageColors
)
from rdap.settings import (
    CACHE_FILE_PATH
)


def decorate(key:str, value:str) -> Union[dict,str]:
    """In charge of decorate the domain status column

    Args:
        key (str): [get the key from the context]
        value (str): [get the value from the context]

    Returns:
        str: [return a user-friendly status]
    """

    if isinstance(value, bool) and key == "status":
        if value:
            value = click.style("Available", fg=MessageColors.GREEN , bold=True)
        else:
            value = click.style("Taken", fg=MessageColors.RED, bold=True)

    elif isinstance(value, bool):
        if value:
            value = click.style("Yes", fg=MessageColors.GREEN , bold=True)
        else:
            value = click.style("No", fg=MessageColors.RED, bold=True)

    
    return value


def get_headers(history:list) -> list:
    """In charge of preparing the table headers.
    Only picks the element 0.

    Args:
        history (list): [this is a list with elements]

    Returns:
        list: [return a list with the elements to be show in the table as headers]
    """
    return [
        head.upper().replace("_", " ") for head in history[0] if not "content" in head
    ]


def get_content(history:list) -> list:
    """In charge of preparing the table rows.

    Args:
        history (list): [this is a list of dicts]

    Returns:
        list: [a list of lists ready to be show in the table]
    """

    body = []
    for objects in history:
        df = [
            decorate(key, value) for key, value in objects.items() if not "content" in key
        ]
        body.append(df)

    return body


def generate_table():
    """Just as the function says, generate a table based on the
    get_headers and get_content functions.

    Returns:
        [type]: [return a table]
    """
    try:
        history = load_file_data(CACHE_FILE_PATH)
    except FileNotFoundError:
        return click.echo(
            click.style(
                "[INFO] - There are no records available yet.",
                fg=MessageColors.YELLOW,
                bold=True,
            )
        )
    
    headers = get_headers(history)
    content = get_content(history)
    return click.echo(
        tabulate(
            content,
            headers,
            tablefmt="fancy_grid",
            stralign="center"
        )
    )


def get_record(id:str) -> dict:
    """In charge of get the specific ID passed from the context
    and return the payload related to it in the history json-file.

    Args:
        id ([str]): [description]

    Returns:
        [tuple[dict|str]]: [
            return a dict if the record was found or a string
            if not
        ]
    """

    try:
        _history = load_file_data(CACHE_FILE_PATH)    
    except FileNotFoundError:
        return click.echo(
            click.style(
                f"[ERROR] - {id} does not exist in your history. ¿Have you deleted it?",
                fg=MessageColors.RED,
                bold=True
            )
        )

    for record in _history:
        if str(id) == str(record["id"]):
            return record

    return click.echo(
        click.style(
            f"[ERROR] - Nothing was found to match with this '{id}'",
            fg=MessageColors.RED,
            bold=True,
        )
    )
